@@ -1,26 +1,6 @@
-/* Spa
- *
- * Copyright © 2018 Wim Taymans
- *
- * Permission is hereby granted, free of charge, to any person obtaining a
- * copy of this software and associated documentation files (the "Software"),
- * to deal in the Software without restriction, including without limitation
- * the rights to use, copy, modify, merge, publish, distribute, sublicense,
- * and/or sell copies of the Software, and to permit persons to whom the
- * Software is furnished to do so, subject to the following conditions:
- *
- * The above copyright notice and this permission notice (including the next
- * paragraph) shall be included in all copies or substantial portions of the
- * Software.
- *
- * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
- * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
- * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
- * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
- * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
- * DEALINGS IN THE SOFTWARE.
- */
+/* Spa */
+/* SPDX-FileCopyrightText: Copyright © 2018 Wim Taymans */
+/* SPDX-License-Identifier: MIT */
 
 #include <errno.h>
 #include <stddef.h>
@@ -42,7 +22,9 @@
 #include <spa/pod/parser.h>
 #include <spa/pod/filter.h>
 
-#define NAME "fakesink"
+#undef SPA_LOG_TOPIC_DEFAULT
+#define SPA_LOG_TOPIC_DEFAULT &log_topic
+SPA_LOG_TOPIC_DEFINE_STATIC(log_topic, "spa.fakesink");
 
 struct props {
 	bool live;
@@ -225,7 +207,7 @@ static inline int read_timer(struct impl *this)
 		if ((res = spa_system_timerfd_read(this->data_system,
 					this->timer_source.fd, &expirations)) < 0) {
 			if (res != -EAGAIN)
-				spa_log_error(this->log, NAME " %p: timerfd error: %s",
+				spa_log_error(this->log, "%p: timerfd error: %s",
 						this, spa_strerror(res));
 		}
 	}
@@ -251,7 +233,7 @@ static int consume_buffer(struct impl *this)
 		spa_node_call_ready(&this->callbacks, SPA_STATUS_NEED_DATA);
 	}
 	if (spa_list_is_empty(&port->ready)) {
-		spa_log_error(this->log, NAME " %p: no buffers", this);
+		spa_log_error(this->log, "%p: no buffers", this);
 		return -EPIPE;
 	}
 
@@ -260,7 +242,7 @@ static int consume_buffer(struct impl *this)
 
 	n_bytes = b->outbuf->datas[0].maxsize;
 
-	spa_log_trace(this->log, NAME " %p: dequeue buffer %d", this, b->id);
+	spa_log_trace(this->log, "%p: dequeue buffer %d", this, b->id);
 
 	render_buffer(this, b);
 
@@ -525,7 +507,7 @@ impl_node_port_enum_params(void *object, int seq,
 static int clear_buffers(struct impl *this, struct port *port)
 {
 	if (port->n_buffers > 0) {
-		spa_log_debug(this->log, NAME " %p: clear buffers", this);
+		spa_log_debug(this->log, "%p: clear buffers", this);
 		port->n_buffers = 0;
 		spa_list_init(&port->ready);
 		this->started = false;
@@ -603,7 +585,7 @@ impl_node_port_use_buffers(void *object,
 		b->h = spa_buffer_find_meta_data(buffers[i], SPA_META_Header, sizeof(*b->h));
 
 		if (d[0].data == NULL) {
-			spa_log_error(this->log, NAME " %p: invalid memory on buffer %p", this,
+			spa_log_error(this->log, "%p: invalid memory on buffer %p", this,
 				      buffers[i]);
 		}
 	}
@@ -655,13 +637,13 @@ static int impl_node_process(void *object)
 		struct buffer *b = &port->buffers[io->buffer_id];
 
 		if (!b->outstanding) {
-			spa_log_warn(this->log, NAME " %p: buffer %u in use", this,
+			spa_log_warn(this->log, "%p: buffer %u in use", this,
 				     io->buffer_id);
 			io->status = -EINVAL;
 			return -EINVAL;
 		}
 
-		spa_log_trace(this->log, NAME " %p: queue buffer %u", this, io->buffer_id);
+		spa_log_trace(this->log, "%p: queue buffer %u", this, io->buffer_id);
 
 		spa_list_append(&port->ready, &b->link);
 		b->outstanding = false;
@@ -838,7 +820,7 @@ impl_enum_interface_info(const struct spa_handle_factory *factory,
 
 const struct spa_handle_factory spa_fakesink_factory = {
 	SPA_VERSION_HANDLE_FACTORY,
-	NAME,
+	"fakesink",
 	NULL,
 	impl_get_size,
 	impl_init,
