@@ -1,26 +1,6 @@
-/* Spa V4l2 Source
- *
- * Copyright © 2018 Wim Taymans
- *
- * Permission is hereby granted, free of charge, to any person obtaining a
- * copy of this software and associated documentation files (the "Software"),
- * to deal in the Software without restriction, including without limitation
- * the rights to use, copy, modify, merge, publish, distribute, sublicense,
- * and/or sell copies of the Software, and to permit persons to whom the
- * Software is furnished to do so, subject to the following conditions:
- *
- * The above copyright notice and this permission notice (including the next
- * paragraph) shall be included in all copies or substantial portions of the
- * Software.
- *
- * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
- * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
- * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
- * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
- * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
- * DEALINGS IN THE SOFTWARE.
- */
+/* Spa V4l2 Source */
+/* SPDX-FileCopyrightText: Copyright © 2018 Wim Taymans */
+/* SPDX-License-Identifier: MIT */
 
 #include <stddef.h>
 #include <sys/types.h>
@@ -43,14 +23,17 @@
 
 #include "v4l2.h"
 
-#define NAME "v4l2-device"
+#undef SPA_LOG_TOPIC_DEFAULT
+#define SPA_LOG_TOPIC_DEFAULT &log_topic
+SPA_LOG_TOPIC_DEFINE_STATIC(log_topic, "spa.v4l2-device");
 
 static const char default_device[] = "/dev/video0";
 
 struct props {
 	char device[64];
-	char product_id[6];
-	char vendor_id[6];
+	char devnum[32];
+	char product_id[7];
+	char vendor_id[7];
 	int device_fd;
 };
 
@@ -75,11 +58,12 @@ struct impl {
 static int emit_info(struct impl *this, bool full)
 {
 	int res;
-	struct spa_dict_item items[12];
+	struct spa_dict_item items[13];
 	uint32_t n_items = 0;
 	struct spa_device_info info;
 	struct spa_param_info params[2];
-        char path[128], version[16], capabilities[16], device_caps[16];
+	char path[128], version[16], capabilities[16], device_caps[16], devices_str[16];
+	struct spa_strbuf buf;
 
 	if ((res = spa_v4l2_open(&this->dev, this->props.device)) < 0)
 		return res;
@@ -98,6 +82,14 @@ static int emit_info(struct impl *this, bool full)
 	if (this->props.vendor_id[0])
 		ADD_ITEM(SPA_KEY_DEVICE_VENDOR_ID, this->props.vendor_id);
 	ADD_ITEM(SPA_KEY_API_V4L2_PATH, (char *)this->props.device);
+
+	/* encode device number into a json array */
+	spa_strbuf_init(&buf, devices_str, sizeof(devices_str));
+	spa_strbuf_append(&buf, "[ ");
+	spa_strbuf_append(&buf, "%s", this->props.devnum);
+	spa_strbuf_append(&buf, " ]");
+	ADD_ITEM(SPA_KEY_DEVICE_DEVIDS, devices_str);
+
 	ADD_ITEM(SPA_KEY_API_V4L2_CAP_DRIVER, (char *)this->dev.cap.driver);
 	ADD_ITEM(SPA_KEY_API_V4L2_CAP_CARD, (char *)this->dev.cap.card);
 	ADD_ITEM(SPA_KEY_API_V4L2_CAP_BUS_INFO, (char *)this->dev.cap.bus_info);
@@ -252,11 +244,13 @@ impl_init(const struct spa_handle_factory *factory,
 	reset_props(&this->props);
 
 	if (info && (str = spa_dict_lookup(info, SPA_KEY_API_V4L2_PATH)))
-		strncpy(this->props.device, str, 63);
+		strncpy(this->props.device, str, sizeof(this->props.device)-1);
+	if (info && (str = spa_dict_lookup(info, SPA_KEY_DEVICE_DEVIDS)))
+		strncpy(this->props.devnum, str, sizeof(this->props.devnum)-1);
 	if (info && (str = spa_dict_lookup(info, SPA_KEY_DEVICE_PRODUCT_ID)))
-		strncpy(this->props.product_id, str, 5);
+		strncpy(this->props.product_id, str, sizeof(this->props.product_id)-1);
 	if (info && (str = spa_dict_lookup(info, SPA_KEY_DEVICE_VENDOR_ID)))
-		strncpy(this->props.vendor_id, str, 5);
+		strncpy(this->props.vendor_id, str, sizeof(this->props.vendor_id)-1);
 
 	return 0;
 }
