@@ -1,26 +1,6 @@
-/* PipeWire
- *
- * Copyright © 2020 Wim Taymans
- *
- * Permission is hereby granted, free of charge, to any person obtaining a
- * copy of this software and associated documentation files (the "Software"),
- * to deal in the Software without restriction, including without limitation
- * the rights to use, copy, modify, merge, publish, distribute, sublicense,
- * and/or sell copies of the Software, and to permit persons to whom the
- * Software is furnished to do so, subject to the following conditions:
- *
- * The above copyright notice and this permission notice (including the next
- * paragraph) shall be included in all copies or substantial portions of the
- * Software.
- *
- * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
- * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
- * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
- * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
- * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
- * DEALINGS IN THE SOFTWARE.
- */
+/* PipeWire */
+/* SPDX-FileCopyrightText: Copyright © 2020 Wim Taymans */
+/* SPDX-License-Identifier: MIT */
 
 #include <stdio.h>
 #include <signal.h>
@@ -33,6 +13,7 @@
 #include <spa/control/control.h>
 #include <spa/param/audio/format-utils.h>
 #include <spa/param/props.h>
+#include <spa/debug/mem.h>
 
 #include <pipewire/pipewire.h>
 #include <pipewire/filter.h>
@@ -65,7 +46,7 @@ static int dump_file(const char *filename)
 		return -1;
 	}
 
-	printf("opened %s\n", filename);
+	printf("opened %s format:%u ntracks:%u division:%u\n", filename, info.format, info.ntracks, info.division);
 
 	while (midi_file_read_event(file, &ev) == 1) {
 		midi_file_dump_event(stdout, &ev);
@@ -96,28 +77,30 @@ static void on_process(void *_data, struct spa_io_position *position)
 	d = &buf->datas[0];
 
 	if (d->data == NULL)
-		return;
+		goto done;
 
 	if ((pod = spa_pod_from_data(d->data, d->maxsize, d->chunk->offset, d->chunk->size)) == NULL)
-		return;
+		goto done;
 	if (!spa_pod_is_sequence(pod))
-		return;
+		goto done;
 
 	SPA_POD_SEQUENCE_FOREACH((struct spa_pod_sequence*)pod, c) {
 		struct midi_event ev;
 
-		if (c->type != SPA_CONTROL_Midi)
+		if (c->type != SPA_CONTROL_UMP)
 			continue;
 
 		ev.track = 0;
 		ev.sec = (frame + c->offset) / (float) position->clock.rate.denom;
 		ev.data = SPA_POD_BODY(&c->value),
 		ev.size = SPA_POD_BODY_SIZE(&c->value);
+		ev.type = MIDI_EVENT_TYPE_UMP;
 
-		printf("%4d: ", c->offset);
+		fprintf(stdout, "%4d: ", c->offset);
 		midi_file_dump_event(stdout, &ev);
 	}
 
+done:
 	pw_filter_queue_buffer(data->in_port, b);
 }
 
@@ -158,7 +141,7 @@ static int dump_filter(struct data *data)
 			PW_FILTER_PORT_FLAG_MAP_BUFFERS,
 			sizeof(struct port),
 			pw_properties_new(
-				PW_KEY_FORMAT_DSP, "8 bit raw midi",
+				PW_KEY_FORMAT_DSP, "32 bit raw UMP",
 				PW_KEY_PORT_NAME, "input",
 				NULL),
 			NULL, 0);
@@ -178,7 +161,7 @@ static int dump_filter(struct data *data)
 
 static void show_help(const char *name, bool error)
 {
-        fprintf(error ? stderr : stdout, "%s [options] [FILE]\n"
+	fprintf(error ? stderr : stdout, "%s [options] [FILE]\n"
 		"  -h, --help                            Show this help\n"
 		"      --version                         Show version\n"
 		"  -r, --remote                          Remote daemon name\n",
